using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using SimpleJSON;
using System.Linq;

namespace ClockwiseSilver
{
	public class SilverCircler : MVRScript
	{
		private JSONStorableBool isActive;
		private JSONStorableFloat radiusJSON;
		private JSONStorableFloat speedXJSON;
		private JSONStorableFloat speedYJSON;
		private JSONStorableStringChooser refAtomJSON;
		private JSONStorableStringChooser targetJSON;
		private JSONStorableStringChooser refTargetJSON;
		
		private FreeControllerV3 controller;
		private Rigidbody controlRB;
		private Atom refAtom;
		private Rigidbody referenceRB;
		private Vector3 controllerStartPos = Vector3.zero;
		private Vector3 currentRot = Vector3.zero;
		private Vector3 refPos = Vector3.zero;
		private bool animateEnabled = false;
		private bool useRef = false;
		
		
		public override void Init()
		{
			try
			{
				if (containingAtom.type != "Person")
				{
					SuperController.LogMessage("SilverCircler: Only works when added to a Person atom.");
					var h = CreateTextField(new JSONStorableString("header0", "Only works when added to a Person atom."),false);
					this.enabled = false;
					return;
				}
				
				isActive = new JSONStorableBool("isActive", false);
				isActive.storeType = JSONStorableParam.StoreType.Full;
				var toggle = CreateToggle(isActive, false);
				toggle.label = "Active";
				RegisterBool(isActive);

				targetJSON = new JSONStorableStringChooser("targetRB", null, null, "Circler", SyncTarget);
				RegisterStringChooser(targetJSON);
				List<string> choices = new List<string>();
				choices.Add("lHand");
				choices.Add("rHand");
				choices.Add("lFoot");
				choices.Add("rFoot");
				choices.Add("lThigh");
				choices.Add("rThigh");
				choices.Add("hip");
				choices.Add("chest");
				choices.Add("neck");
				choices.Add("abdomen");
				choices.Add("abdomen2");
				targetJSON.choices = choices;
				var dp = CreateFilterablePopup(targetJSON);
				dp.popupPanelHeight = 700f;
				
				var spacee = CreateSpacer();
				spacee.height = 20f;
				var heading = CreateTextField(new JSONStorableString("header1",
				"Reference Atom: Optional\nMaintains the Circler's position\nrelative to another body."),false);
                heading.height = 40f;	
				
				//RefAtom
				refAtomJSON = new JSONStorableStringChooser("refAtom", SuperController.singleton.GetAtomUIDs(), null, "Ref Atom", RefTargetChoices);
				RegisterStringChooser(refAtomJSON);
				AtomChocies();
				dp = CreateFilterablePopup(refAtomJSON);
				dp.popupPanelHeight = 700f;
				dp.popup.onOpenPopupHandlers += AtomChocies;

				//RefTarget
				refTargetJSON = new JSONStorableStringChooser("refTarget", null, null, "Ref Target", SyncRefTarget);
				RegisterStringChooser(refTargetJSON);
				dp = CreateFilterablePopup(refTargetJSON);
				dp.popupPanelHeight = 600f;
				
				radiusJSON = NewSlider("Radius", 50f, 1f, 400f, true);
				speedXJSON = NewSlider("Speed X", 0f, -500f, 500f, true);
				speedYJSON = NewSlider("Speed Y", 0f, -500f, 500f, true);				
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
		private void Start()
		{
			isActive.toggle.onValueChanged.AddListener(checkVal =>
				{
					if (checkVal) { StartCircler(); }
					else { StopCircler(); }
				});
			if (isActive.val) { StartCircler(); }
		}
		
		private void StartCircler()
		{
			if (controller == null || controlRB == null)
			{
				SuperController.LogMessage("SilverCircler: The circler is not a valid target.");
				animateEnabled = false;
				isActive.val = false;
				return;
			}
			else
			{
				currentRot = Vector3.zero;
				controllerStartPos = controller.transform.position;
				if (referenceRB != null && refTargetJSON.val != "None")
				{
					useRef = true;
					refPos = referenceRB.transform.InverseTransformPoint(controller.transform.position);
				}
				else
				{
					useRef = false;
					refPos = controller.transform.position;
				}
				animateEnabled = true;
			}
		}
		
		private void StopCircler()
		{
			if (!animateEnabled) { return; }
			controller.transform.position = controllerStartPos;
			animateEnabled = false;
		}
		
		private void FixedUpdate()
		{
			try
			{
				if (animateEnabled)
				{
					currentRot.x += speedXJSON.val * Time.fixedDeltaTime;
					if (currentRot.x >= 360f) { currentRot.x = 0f; }
					else if (currentRot.x <= 0f) { currentRot.x = 360f; }
					currentRot.y += speedYJSON.val * Time.fixedDeltaTime;
					if (currentRot.y >= 360f) { currentRot.y = 0f; }
					else if (currentRot.y <= 0f) { currentRot.y = 360f; }					
					
					Vector3 dir = controller.transform.TransformDirection(Quaternion.Euler(currentRot) * Vector3.forward);
					if (useRef)
					{
						controller.transform.position = referenceRB.transform.TransformPoint(refPos);
					}
					controlRB.AddForce(dir * radiusJSON.val, ForceMode.Force);
				}
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}			
		}
		
		private void SyncTarget(string targetID)
		{
			if (animateEnabled) { isActive.val = false; }
			if (targetID != null && targetID != "" && targetID != "None")
			{
				controlRB = containingAtom.rigidbodies.First(rb => rb.name == targetID);
				controller = containingAtom.freeControllers.First(fc => fc.name == targetID + "Control");
			}
			else
			{
				controlRB = null;
				controller = null;
			}
		}
		
		private void AtomChocies()
		{
			if (animateEnabled) { isActive.val = false; }
			List<string> atomChoices = new List<string>();
			atomChoices.Add("None");
			foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
			{
				atomChoices.Add(atomUID);
			}
			refAtomJSON.choices = atomChoices;
		}
		
		private void RefTargetChoices(string atomUID)
		{
			if (animateEnabled) { isActive.val = false; }
			List<string> choices = new List<string>();
			choices.Add("None");
			if (atomUID != null)
			{
				refAtom = SuperController.singleton.GetAtomByUid(atomUID);
				if (refAtom != null)
				{
					foreach (Rigidbody rb in refAtom.rigidbodies)
					{
						if (!rb.name.Contains("Collider") && !rb.name.Contains("Control") && !rb.name.Contains("Link"))
						{
							choices.Add(rb.name);
						}
					}
				}
			}
			else
			{
				refAtom = null;
			}
			refTargetJSON.choices = choices;
		}
		
		private void SyncRefTarget(string refTargetID)
		{
			if (animateEnabled) { isActive.val = false; }
			if (refTargetID != null && refTargetID != "" && refTargetID != "None")
			{
				referenceRB = refAtom.rigidbodies.First(rb => rb.name == refTargetID);
			}
			else
			{
				referenceRB = null;
			}
		}
		
		private JSONStorableFloat NewSlider(string name, float val, float min, float max, bool right)
		{
			JSONStorableFloat storable = new JSONStorableFloat(name, val, min, max, true, true);
			storable.storeType = JSONStorableParam.StoreType.Full; CreateSlider(storable, right); RegisterFloat(storable);
			return storable;
		}
	}
}